/*
* Copyright 1999-2004 The Apache Software Foundation.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


// ----------------------------------------------------------------
//  This file was generated from the XalanC error message source.
//  so do not edit this file directly!!
// ----------------------------------------------------------------




#if !defined(XALAN_MSG_LOADER_INDEX_GUARD_1357924680) 
#define XALAN_MSG_LOADER_INDEX_GUARD_1357924680 



namespace XALAN_CPP_NAMESPACE {


class XalanMessages{ 
public : 
    enum Codes 
   { 
      SystemErrorCode_1Param		 = 0 
     ,UnsupportedEncoding_1Param		 = 1 
     ,CreateTranscoderError_2Param		 = 2 
     ,UnknownErrorOccurredWhileTranscodingToEncoding_1Param		 = 3 
     ,AnErrorOccurredWhileTranscoding		 = 4 
     ,MessageErrorCodeWas_1Param		 = 5 
     ,FunctionRequiresNonNullContextNode_1Param		 = 6 
     ,FunctionTakesTwoArguments_1Param		 = 7 
     ,FunctionTakesZeroOrOneArg_1Param		 = 8 
     ,FunctionTakesTwoOrThreeArguments_1Param		 = 9 
     ,FunctionTakesThreeArguments_1Param		 = 10 
     ,CannotConvertTypetoType_2Param		 = 11 
     ,ExpressionDoesNotEvaluateToNodeSet		 = 12 
     ,UnknownAxis_1Param		 = 13 
     ,CannotEvaluateXPathExpressionAsMatchPattern		 = 14 
     ,ArgLengthNodeTestIsIncorrect_1Param		 = 15 
     ,InvalidOpcodeWasDetected_1Param		 = 16 
     ,InvalidNumberOfArgsWasDetected_3Param		 = 17 
     ,InvalidNumberOfArgsWasSupplied_2Param		 = 18 
     ,RemainingTokens		 = 19 
     ,FunctionIsNotImplemented_1Param		 = 20 
     ,FunctionNumberIsNotAvailable_1Param		 = 21 
     ,FunctionIsNotSupported_1Param		 = 22 
     ,ExtraIllegalTokens		 = 23 
     ,UnterminatedStringLiteral		 = 24 
     ,EmptyExpression		 = 25 
     ,NotValidNCName_1Param		 = 26 
     ,PrefixIsBoundToZeroLengthURI_1Param		 = 27 
     ,PrefixIsNotDeclared_1Param		 = 28 
     ,NotFoundWhatExpected_2Param		 = 29 
     ,LiteralArgumentIsRequired		 = 30 
     ,NoPrecedingArgument		 = 31 
     ,NoFollowingArgument		 = 32 
     ,CouldNotFindFunction_1Param		 = 33 
     ,FunctionDoesNotAcceptAnyArguments_1Param		 = 34 
     ,IllegalSyntaxOfPredicatesSelf		 = 35 
     ,IllegalSyntaxOfPredicatesParent		 = 36 
     ,UnexpectedTokenFound_1Param		 = 37 
     ,ExpectedAxis		 = 38 
     ,IllegalAxisName_1Param		 = 39 
     ,UnknownNodeType_1Param		 = 40 
     ,ExpectedNodeTest		 = 41 
     ,OnlyChildAndAttributeAxesAreAllowed		 = 42 
     ,VariableReferenceNotAllowed		 = 43 
     ,KeyFunctionNotAllowed		 = 44 
     ,FunctionTakes2ArgsAtLeast_1Param		 = 45 
     ,PatternIs_1Param		 = 46 
     ,ExpressionIs_1Param		 = 47 
     ,SpecifiedFuncIsNotAvailable_1Param		 = 48 
     ,AttributesCannotBeAdded		 = 49 
     ,IllegalElementName_1Param		 = 50 
     ,XalanHandleExtensions		 = 51 
     ,CannotResolveURIInDocumentFunction		 = 52 
     ,UnknownXSLElement_1Param		 = 53 
     ,NameIs_1Param		 = 54 
     ,AttributeCannotBeAdded		 = 55 
     ,CantCreateItemInResultTree		 = 56 
     ,LeftBraceCannotAppearWithinExpression		 = 57 
     ,AttributeValueTemplateHasMissing		 = 58 
     ,NoCurrentTemplate		 = 59 
     ,AttributeNameNotValidQName_1Param		 = 60 
     ,IsNotValidQName_1Param		 = 61 
     ,IsNotValidNCName_1Param		 = 62 
     ,AttributeValueNotValidQName_2Param		 = 63 
     ,ElementMustHaveAttribute_2Param		 = 64 
     ,CannotFindNamedTemplate		 = 65 
     ,ElementHasIllegalAttributeValue_3Param		 = 66 
     ,DuplicateDefinitions_1Param		 = 67 
     ,MustBeOrPrefixedName		 = 68 
     ,UnknownDataType		 = 69 
     ,GroupingSeparatorValueMustBeOneCharacterLength		 = 70 
     ,NumberingFormatNotSupported_1Param		 = 71 
     ,PINameInvalid_1Param		 = 72 
     ,NameMustBeValidNCName		 = 73 
     ,ElementRequiresEitherNameOrMatchAttribute_1Param		 = 74 
     ,ElementHasIllegalAttribute_2Param		 = 75 
     ,ElementIsNotAllowedAtThisPosition_1Param		 = 76 
     ,ElemVariableInstanceAddedToWrongStylesheet		 = 77 
     ,ElemVariableInstanceIsAlreadyParented		 = 78 
     ,SecondArgumentToFunctionMustBeNode_set_1Param		 = 79 
     ,FunctionAcceptsOneTwoArguments_1Param		 = 80 
     ,Decimal_formatElementNotFound_1Param		 = 81 
     ,PropertyIsNotValidQName_1Param		 = 82 
     ,InvalidArgumentType_1Param		 = 83 
     ,FunctionAcceptsOneArgument_1Param		 = 84 
     ,XSLTError		 = 85 
     ,XSLTWarning		 = 86 
     ,XSLTMessage		 = 87 
     ,XMLError		 = 88 
     ,XMLWarning		 = 89 
     ,XMLMessage		 = 90 
     ,XPathError		 = 91 
     ,XPathWarning		 = 92 
     ,XPathMessage		 = 93 
     ,StyleTreeNode_1Param		 = 94 
     ,SourceTreeNode_1Param		 = 95 
     ,LineNumberColumnNumber_2Params		 = 96 
     ,HasInvalidAttribute_2Param		 = 97 
     ,AttributeMustBe_3Params		 = 98 
     ,StylesheetHasWrapperlessTemplate		 = 99 
     ,StylesheetHasDuplicateNamedTemplate_1Param		 = 100 
     ,UnknownXSLTToken_1Param		 = 101 
     ,InfiniteRecursion_1Param		 = 102 
     ,VariableIsNotDefined_1Param		 = 103 
     ,IsNotAllowedInThisPosition_1Param		 = 104 
     ,NotParentedBy_2Param		 = 105 
     ,IsNotAllowedInsideTemplate_1Param		 = 106 
     ,StylesheetAttribDidNotSpecifyVersionAttrib		 = 107 
     ,VariableHasBeenDeclared		 = 108 
     ,VariableHasBeenDeclaredInThisTemplate		 = 109 
     ,ImportCanOnlyOccur		 = 110 
     ,ImportingItself_1Param		 = 111 
     ,IncludingItself_1Param		 = 112 
     ,CharIsNotAllowedInStylesheet		 = 113 
     ,Transform		 = 114 
     ,NoValidResultTarget		 = 115 
     ,OutputHasAnUnknownMethod_1Param		 = 116 
     ,UnsupportedXalanSpecificAttribute_1Param		 = 117 
     ,HasIllegalAttribute_2Param		 = 118 
     ,TextAndColumnNumber_2Param		 = 119 
     ,MatchIs_1Param		 = 120 
     ,Warning2		 = 121 
     ,FatalError		 = 122 
     ,FormatNumberFailed		 = 123 
     ,UnknownMatchOpCode_1Param		 = 124 
     ,InEntity_1Param		 = 125 
     ,InEntity_3Param		 = 126 
     ,InUnknownEntity_2Param		 = 127 
     ,InUnknownEntity		 = 128 
     ,EmptyNodeList		 = 129 
     ,DefaultRootRule_1Param		 = 130 
     ,DefaultRootRule		 = 131 
     ,InvalidStackContext		 = 132 
     ,FailedToProcessStylesheet		 = 133 
     ,TotalTime		 = 134 
     ,NoStylesheet		 = 135 
     ,CompilingDOMStylesheetReqDocument		 = 136 
     ,CantIdentifyFragment_1Param		 = 137 
     ,CantFindFragment_1Param		 = 138 
     ,NodePointedByFragment_1Param		 = 139 
     ,OnlyTextNodesCanBeCopied		 = 140 
     ,UnmatchedWasFound		 = 141 
     ,CantLoadReqDocument_1Param		 = 142 
     ,CantLoadReqDocument_2Param		 = 143 
     ,ElementRequiresAttribute_2Param		 = 144 
     ,InputXSL		 = 145 
     ,ConflictsFound		 = 146 
     ,CircularVariableDefWasDetected		 = 147 
     ,FunctionAcceptsTwoArguments_1Param		 = 148 
     ,NoPrefixResolverAvailable		 = 149 
     ,InvalidArgumentTypeFunction_1Param		 = 150 
     ,ErrorCopyingNamespaceNodeForDefault		 = 151 
     ,ErrorCopyingNamespaceNode_1Param		 = 152 
     ,InvalidHighSurrogate_1Param		 = 153 
     ,InvalidSurrogatePair_2Param		 = 154 
     ,EXSLTFunctionAcceptsOneArgument_1Param		 = 155 
     ,EXSLTFunctionAcceptsNoArgument_1Param		 = 156 
     ,EXSLTFunctionAcceptsTwoArguments_1Param		 = 157 
     ,EXSLTFunctionAcceptsTwoOrThreeArguments_1Param		 = 158 
     ,EXSLTFunctionAccepts1Or2Argument_1Param		 = 159 
     ,XalanDOMExceptionCaught_1Param		 = 160 
     ,InvalidCompiledStylesheetProvided		 = 161 
     ,InvalidParsedSourceProvided		 = 162 
     ,NumberBytesWrittenDoesNotEqual		 = 163 
     ,XalanExeHelpMenuXalanVersion_1Param		 = 164 
     ,XalanExeHelpMenuXercesVersion_1Param		 = 165 
     ,XalanExeHelpMenu		 = 166 
     ,XalanExeHelpMenu0		 = 167 
     ,XalanExeHelpMenu1		 = 168 
     ,XalanExeHelpMenu2		 = 169 
     ,XalanExeHelpMenu3		 = 170 
     ,XalanExeHelpMenu4		 = 171 
     ,XalanExeHelpMenu5		 = 172 
     ,XalanExeHelpMenu6		 = 173 
     ,XalanExeHelpMenu7		 = 174 
     ,XalanExeHelpMenu8		 = 175 
     ,XalanExeHelpMenu9		 = 176 
     ,XalanExeHelpMenu10		 = 177 
     ,XalanExeHelpMenu11		 = 178 
     ,XalanExeHelpMenu12		 = 179 
     ,ElemOrLTIsNotAllowed_1Param		 = 180 
     ,ElemIsNotAllowed_1Param		 = 181 
     ,ErrorWritingFile_1Param		 = 182 
     ,ErrorOpeningFile_1Param		 = 183 
     ,Error2		 = 184 
     ,ErrorWritingToStdStream		 = 185 
     ,UnrepresentableCharacter_2Param		 = 186 
     ,InvalidScalar_1Param		 = 187 
     ,PrefixOfLengthZeroDetected		 = 188 
     ,SortMustBeAscendOrDescend		 = 189 
     ,SortDataTypeMustBe		 = 190 
     ,SortHasUnknownDataType		 = 191 
     ,SortCaseOrderMustBe		 = 192 
     ,LegalValuesForLetterValue		 = 193 
     ,InvalidURI		 = 194 
     ,ExpectedToken		 = 195 
     ,ForbiddenXMLCharacter_2Param		 = 196 
     ,UnknownKey_1Param		 = 197 
     ,CannotSwitchToHTMLOutputMethod		 = 198 
     ,CannotFindFragment_2Param		 = 199 
   };


};


} 


#endif //XALAN_MSG_LOADER_INDEX_GUARD_1357924680  


